#!perl

require 't/scripts/fib.pl';

fib( 1 );

# The expected output
__DATA__
ENTRY(fib, t/scripts/fib.pl, 9)
ENTRY(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
ENTRY(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 6)
RETURN(fib, t/scripts/fib.pl, 9)
ENTRY(fib, t/scripts/fump.pl, 5)
RETURN(fib, t/scripts/fump.pl, 5)
